using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Net;
using System.Net.NetworkInformation;
using System.Diagnostics;
using LibJasinski.Network;

namespace DesktopText
{
	class VerseGetter
	{
		private static string _sourceUrl;
		private static string _verse = "";
		private static string _reference = "";
		private static InternetConnector _conn;
		private static string _proxyServer;

		public static event NoArgsDelegate ProxyStateChanged;

		public const string NotApplicable = "(n/a)";

		static VerseGetter()
		{
			_conn = new InternetConnector("Desktop Text");
			_conn.ProxyStateChanged += new NoArgsDelegate(_conn_ProxyStateChanged);
			_sourceUrl = Cfg.Def.SourceUrl;
		}

		static void _conn_ProxyStateChanged()
		{
			if (VerseGetter.ProxyStateChanged != null)
				VerseGetter.ProxyStateChanged();
		}

		public static string Verse
		{
			get {return _verse; }
		}

		public static string Reference
		{
			get { return _reference; }
		}

		public static string ProxyServer
		{
			get { return _proxyServer; }
		}

		public static ProxyState ProxyState
		{
			get { return _conn.ProxyState; }
		}

		public static bool Get()
		{
			bool result = false;
			bool tryAgain = true;
			WebClient wc;
			string html = "";

            try
            {
				while (tryAgain)
				{
					wc = _conn.GetConnection();

					if (_conn.ProxyState == ProxyState.NotUsing)
						_proxyServer = NotApplicable;
					else
						_proxyServer = _conn.ProxyServer;

					if (wc != null)
					{
						try
						{
							html = wc.DownloadString(_sourceUrl);
							_conn.ProxyState = ProxyState.Verified;
							tryAgain = false;
						}
						catch (Exception ex)
						{
							Debug.Print(ex.Message);
							if (_conn.ProxyState == ProxyState.NotUsing)
								tryAgain = false;
						}
					}
				}


				if (html.Length > 0)
				{
					string pattern = Cfg.Def.RegexPattern;
					Regex regex = new Regex(pattern);
					MatchCollection mc = regex.Matches(html);
					int n = mc.Count;
					if (n > 0)
					{
						_verse = InsertLineBreaks(mc[0].Groups["verse"].Value);
						_reference = mc[0].Groups["reference"].Value;
						result = _verse.Length > 0 & _reference.Length > 0;
					}
				}
				return result;
            }
            catch (Exception ex)
            {
				Debug.Print(ex.Message);
                return false;
            }
		}
		private static string InsertLineBreaks(string input)
		{
			int maxLineLength = 100;
			int lineLength = 0;
			string result = "";
			string[] words = input.Split(new char[] {' '});

			for (int i = 0; i <= words.GetUpperBound(0); i++)
			{
				if (lineLength + words[i].Length > maxLineLength)
				{
					lineLength = 0;
					result += Environment.NewLine;
				}
				lineLength += words[i].Length + 1;
				result += words[i];
				result += " ";
			}
			return result;
		}
	}
}
